package	jp.co.sds_corp.hibiya.server;

import java.text.DecimalFormat;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.device.DeviceManager;
import jp.co.sds_corp.util.device.DriveInfo;

/**
ディスクの状況を監視します。
<BR>
XMLは以下の形式です。
<PRE>
&lt;Service name="Drive1" class="jp.co.sds_corp.hibiya.server.DriveChecker"&gt;
	&lt;Drive&gt;/home&lt;/Drive&gt;
	&lt;Minimum&gt;20&lt;/Minimum&gt;
</PRE>
Minimum要素では、最小の残り容量をパーセントで指定してください。
ディスクの残り容量（パーセント）がこの値より小さくなるとエラーです。<BR>
<BR>
Action要素については、{@link jp.co.sds_corp.hibiya.action.IAction}
の説明を参照してください。
*/
public class DriveChecker extends AbstractChecker
{
	private static final long serialVersionUID = 1L;
	DecimalFormat	df = new DecimalFormat( "#,###" );
	DecimalFormat	df2 = new DecimalFormat( "#.0#" );
	String	drive;
	int		minimum;

	public DriveChecker() {
		setType( "Drive" );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		drive = config.get( "Drive" );
		minimum = config.getInteger( "Minimum" );
	}

	/**
	* 監視処理を実行します。
	*/
	public void check() {
		try {
			CmnDate startTime = new CmnDate();
			DriveInfo info = DeviceManager.getDriveInfo( drive );
			CmnDate endTime = new CmnDate();
			clearResult();
			setResponseTime( startTime, endTime );

			putResult( "Name", info.getName() );
			putResult( "Device", info.getDeviceName() );
			putResult( "Filesystem", info.getFileSystem() );
			putResult( "MaxSize", df.format( info.getMaxSize() ));
			putResult( "UsedSize", df.format( info.getUsedSize() ));
			putResult( "AvailSize", df.format( info.getAvailSize() ));
			putResult( "FreeSize", df.format( info.getFreeSize() ));
			putResult( "BlockSize", df.format( info.getBlockSize() ));

			double total = (double )( info.getAvailSize() + info.getUsedSize());
			double rest = (double )info.getAvailSize() / total * 100.0;
			if ( total == 0.0 ) {
				putResult( "rest", "0%" );
			} else {
				putResult( "rest", df2.format( rest ) + "%" );
			}

			if ( rest < minimum ) {
				setException( new Exception( "no more disk [" + drive + "]"
								+ " rest=" + df2.format( rest )));
				setCheckerStatus( IChecker.Status.NG );
			} else {
				setCheckerStatus( IChecker.Status.OK );
			}
		} catch ( Throwable ex ) {
			synchronized( this ) {
				clearResult();
				setException( ex );
				setCheckerStatus( IChecker.Status.NG );
			}
		}
	}

	/**
	* ドライブ名(drive)を設定します。
	* @param	var		設定するデータ
	*/
	public void setDrive( String var ) {
		this.drive = var ;
	}
	/**
	* ドライブ名(drive)を取得します。
	* @return	driveの内容
	*/
	public String getDrive() {
		return drive;
	}

	/**
	* 最小使用可能サイズ(minimum)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMinimum( int var ) {
		this.minimum = var ;
	}
	/**
	* 最小使用可能サイズ(minimum)を取得します。
	* @return	minimumの内容
	*/
	public int getMinimum() {
		return minimum;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "Drive", drive );
		config.addChild( "Minimum", String.valueOf( minimum ));
	}

	/**
	* このモニタの文字列表現を返します。
	* @return	このモニタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += "["
			+ "drive=" + drive + ";"
			+ "min=" + minimum + ";"
			+ "]";
		return ret;
	}

}
